@@ -77,7 +77,7 @@ class TmuxSession:
         if not asciinema_installed:
             tools_needed.append("asciinema")
 
-        self._logger.info(f"Installing: {', '.join(tools_needed)}")
+        self._logger.debug(f"Installing: {', '.join(tools_needed)}")
 
         # Detect system and package manager
         system_info = await self._detect_system_info()
@@ -249,7 +249,7 @@ class TmuxSession:
                 command="tmux -V || /usr/local/bin/tmux -V"
             )
             if verify_result.return_code == 0:
-                self._logger.info("tmux successfully built and installed from source")
+                self._logger.debug("tmux successfully built and installed from source")
             else:
                 self._logger.error("Failed to install tmux from source")
 
@@ -286,7 +286,7 @@ class TmuxSession:
                         command="asciinema --version"
                     )
                     if verify_result.return_code == 0:
-                        self._logger.info("asciinema successfully installed using pip")
+                        self._logger.debug("asciinema successfully installed using pip")
                         return
 
             self._logger.error("Failed to install asciinema using pip")
@@ -393,7 +393,7 @@ class TmuxSession:
                     self._markers, self._local_asciinema_recording_path
                 )
                 handler.merge_markers()
-                self._logger.info(
+                self._logger.debug(
                     f"Successfully merged markers into {self._local_asciinema_recording_path}"
                 )
             else:
