@@ -1,6 +1,7 @@
 from typer import Typer
 
 from harbor.cli.admin.admin import admin_app
+from harbor.cli.cache import cache_app
 from harbor.cli.datasets import datasets_app
 from harbor.cli.jobs import jobs_app, start
 from harbor.cli.sweeps import sweeps_app
@@ -17,6 +18,7 @@ app.add_typer(traces_app, name="traces", help="Trace export utilities.")
 app.add_typer(
     sweeps_app, name="sweeps", help="Run successive sweeps to focus on successes."
 )
+app.add_typer(cache_app, name="cache", help="Manage Harbor cache.")
 app.add_typer(admin_app, name="admin")
 
 app.command(name="run", help="Start a job. Alias for `harbor jobs start`.")(start)
