@@ -77,8 +77,8 @@ class E2BEnvironment(BaseEnvironment):
         wait=wait_exponential(multiplier=1, min=1, max=10),
         reraise=True,
     )
-    async def _create_template(self, force_build: bool):
-        if self.task_env_config.docker_image and not force_build:
+    async def _create_template(self):
+        if self.task_env_config.docker_image:
             template = Template().from_image(
                 image=self.task_env_config.docker_image,
             )
@@ -111,7 +111,7 @@ class E2BEnvironment(BaseEnvironment):
             timeout=86_400,
         )
 
-    async def _does_sandbox_exist(self) -> bool:
+    async def _does_template_exist(self) -> bool:
         process = await asyncio.create_subprocess_exec(
             "e2b",
             "template",
@@ -132,10 +132,10 @@ class E2BEnvironment(BaseEnvironment):
         }
 
     async def start(self, force_build: bool):
-        if force_build or not await self._does_sandbox_exist():
-            print(f"Creating template {self._template_name}")
+        if force_build or not await self._does_template_exist():
+            self.logger.info(f"Creating template {self._template_name}")
 
-            await self._create_template(force_build=force_build)
+            await self._create_template()
 
         await self._create_sandbox()
 
@@ -159,7 +159,7 @@ class E2BEnvironment(BaseEnvironment):
     async def stop(self, delete: bool):
         """Stops the environment and optionally deletes it."""
         if not delete:
-            print(
+            self.logger.info(
                 "E2B harbor are ephemeral and will be deleted after use, "
                 "regardless of delete=False."
             )
@@ -168,11 +168,11 @@ class E2BEnvironment(BaseEnvironment):
             try:
                 await self._stop_sandbox()
             except Exception as e:
-                print(f"Error stopping sandbox: {e}")
+                self.logger.error(f"Error stopping sandbox: {e}")
             finally:
                 self._sandbox = None
         else:
-            print("Sandbox has already been removed.")
+            self.logger.info("Sandbox has already been removed.")
 
     @retry(
         stop=stop_after_attempt(2),
