@@ -2,6 +2,8 @@ from typing import Any
 
 from pydantic import BaseModel, Field
 
+from harbor.models.agent.rollout_detail import RolloutDetail
+
 
 class AgentContext(BaseModel):
     n_input_tokens: int | None = Field(
@@ -16,11 +18,12 @@ class AgentContext(BaseModel):
     cost_usd: float | None = Field(
         default=None, description="The cost in USD for the agent execution."
     )
-    output_logprobs: list[list[float]] | None = Field(
+    rollout_details: list[RolloutDetail] | None = Field(
         default=None,
         description=(
-            "Log probabilities for each assistant message. Each element is a "
-            "list of logprobs for tokens in that message."
+            "Detailed information about each rollout trajectory including token IDs, "
+            "loss masks, and logprobs. Each element represents one trajectory. For a "
+            "linear chat history, there is only one rollout trajectory."
         ),
     )
     metadata: dict[str, Any] | None = Field(
